#!/usr/bin/env ruby
# encoding: UTF-8

# usage:
# ruby ./createNavigation.rb /home/user/htdocs/ something/index.php

require 'pathname'
# require 'rubygems'
# require 'pry'
require 'rexml/document'

include REXML

class NavigationItem

  attr_accessor :name
  attr_accessor :link
  attr_accessor :other_links
  attr_accessor :sub_links
  attr_accessor :directory
  attr_accessor :parent_directory
  attr_accessor :indent
  attr_accessor :level
  attr_accessor :rootElement

  attr_accessor :indentation
  attr_accessor :spaces
  attr_accessor :collapseString

  attr_accessor :currentSite
  attr_accessor :currentParent

  def initialize(level = 0, indentation = 0)
    @sub_links = []
    @other_links = []
    @level = level
    @indentation = indentation
    @indent = @indentation + (level * 2) - 1
    if level == 0
      @indent = @indentation
    end

    @collapseString = []
    @collapseString[0] = ""
    @collapseString[1] = " <small class=\"arrow\">▼</small></a>\n"
    @collapseString[2] = "&nbsp;&raquo;</a>\n"

    @spaces = " "
    @currentSite = false
    @currentParent = false
  end

  def add(sub_link)
    @sub_links << sub_link
  end

  def checkFilename(filename)
    if "#{@parent_directory}#{@link}" == filename
      return true
    end
    @other_links.each do |other_link|
      if "#{@parent_directory}#{other_link}" == filename
        return true
      end
    end
    return false
  end

  def currentParent=(value)
    # puts " === currentParent #{@rootElement}"
    @currentParent = value
    if @rootElement != nil
      @rootElement.currentParent = true
    end
  end

  def currentSite=(value)
    # puts " === currentSite #{@rootElement}"
    @currentSite = value
    @rootElement.currentParent = true
  end

  def to_s
    linkString = ""
    @indent.times { linkString <<= "#{@spaces}" }
    if @currentSite == true
      linkString <<= "<li class=\"currentSite\"><a>#{@name}"
    else
      if @currentParent == true
        linkString <<= "<li class=\"currentSite\"><a href=\"#{@parent_directory}#{@link}\">#{@name}"
      else
        linkString <<= "<li><a href=\"#{@parent_directory}#{@link}\">#{@name}"
      end
    end

    if (@sub_links != []) and (@level < MyNavigation.maximum_level)

      # add special characters to the menu
      if MyNavigation.sitemap == false
        linkString <<= @collapseString[@level]
      else
        linkString <<= "</a>\n"
      end

      @indent.times { linkString <<= "#{@spaces}" }
      linkString <<= "#{@spaces}<ul>\n"
      @sub_links.each do |sub_link|
        linkString <<= "#{sub_link}"
      end
      @indent.times { linkString <<= "#{@spaces}" }
      linkString <<= "#{@spaces}</ul>\n"

      @indent.times { linkString <<= "#{@spaces}" }
      linkString <<= "</li>\n"
    else
      linkString <<= "</a></li>\n"
    end

=begin
    if @other_links != []
      @other_links.each do |other_link|
        linkString <<= "=== other link: #{@parent_directory}#{other_link}\n"
      end
    end
=end

    return linkString
  end

end

# special root element of navigation
class NavigationRoot < NavigationItem

  def to_s
    linkString = ""
    @indentation.times { linkString <<= "#{@spaces}" }
    if MyNavigation.sitemap == true
      linkString <<= "<ul class=\"bottomMenu\">\n"
    else
      linkString <<= "<ul class=\"dropdown\">\n"
    end
    @sub_links.each do |sub_link|
      linkString <<= "#{sub_link}"
    end
    @indentation.times { linkString <<= "#{@spaces}" }
    linkString <<= '</ul>'
    return linkString
  end

end

module Navigation

  INDEX_FILENAME=".index.xml"

  def self.loadNavigation(base_directory, directory, level, rootElement)

    xml_file = File.new(File.join(base_directory, directory, INDEX_FILENAME))
    xml_document = Document.new(xml_file)

    # set the default filename in a directory:
    if level == 1
      XPath.each(xml_document, "//menu/defaultFile") { |element|
        MyNavigation.defaultFilename = element.attribute("name").to_s
      }
    end

    XPath.each(xml_document, "//menu/entry") { |element|
      mylink = NavigationItem.new(level, rootElement.indentation)
      mylink.spaces = rootElement.spaces
      mylink.collapseString = rootElement.collapseString
      mylink.rootElement = rootElement

      mylink.name = element.get_elements("name")[0].text
      temporarylink = element.get_elements("link")[0].text

      if File.directory?(File.join(base_directory, directory, temporarylink))
        mylink.link = File.join(temporarylink, MyNavigation.defaultFilename)
        mylink.directory = File.join(directory, temporarylink)
      else
        mylink.link = temporarylink
      end

      # add alternative links to mylink
      element.get_elements('otherlink').each { |myelement|
        mylink.other_links << myelement.text
      }

      mylink.parent_directory = directory

      if File.exist?(File.join(base_directory, directory, temporarylink, INDEX_FILENAME))
        loadNavigation(base_directory, directory + temporarylink + File::SEPARATOR, level + 1, mylink)
      end

      rootElement.add(mylink)
    }

  end

  def self.checkLink(filename, rootElement, level)

    # puts "in checkLink #{level} #{rootElement.link}"

    if rootElement.checkFilename(filename)
      # puts "Found: #{rootElement.link}\n\n"
      rootElement.currentSite = true
      MyNavigation.currentElement = rootElement
      return true
    end

    rootElement.sub_links.each do |sub_link|
      if checkLink(filename, sub_link, level + 1) == true
        return true
      end
      if sub_link.checkFilename(filename)
        # puts "Found: #{sub_link.link}\n\n"
        sub_link.currentSite = true
        MyNavigation.currentElement = sub_link
        return true
      end
    end

    return false
  end

end

class MyNavigation

  @@currentElement = nil
  @@defaultFilename="index.php"
  @@defaultExtension="php"
  @@maximum_level=3
  @@sitemap=false

  def self.sitemap=(sitemap)
    @@sitemap=sitemap
  end

  def self.sitemap
    @@sitemap
  end

  def self.maximum_level=(maximum_level)
    @@maximum_level=maximum_level
  end

  def self.maximum_level
    @@maximum_level
  end

  def self.defaultExtension
    @@defaultExtension
  end

  def self.currentElement
    @@currentElement
  end

  def self.currentElement=(currentElement)
    @@currentElement = currentElement
  end

  def self.defaultFilename
    @@defaultFilename
  end

  def self.defaultFilename=(defaultFilename)
    @@defaultFilename = defaultFilename
    @@defaultExtension = defaultFilename.sub(/.*\./, "")
  end

end

# change the directory
module DirectoryStructure

  def self.modifyDirectories(htmlString, directoryDepth, file_name)

    htmlString.gsub!(/^ include '.*log\/logging.php';$/, " include '#{directoryDepth}log/logging.php';")

    htmlString.gsub!(/^ phpLog\(".*", ".*log\/"\);$/, " phpLog(\"#{file_name}\", \"#{directoryDepth}log/\");")

    htmlString.gsub!(/^<link href=".*style\/general.css" rel="stylesheet" type="text\/css" \/>$/,
      "<link href=\"#{directoryDepth}style/general.css\" rel=\"stylesheet\" type=\"text/css\" />")

    htmlString.gsub!(/^<link rel="stylesheet" type="text\/css" href=".*style\/ie6.css" media="screen" \/>$/,
      "<link rel=\"stylesheet\" type=\"text/css\" href=\"#{directoryDepth}style\/ie6.css\" media=\"screen\" />")

    htmlString.gsub!(/^<script type="text\/javascript" src=".*javascript\/ie6dropdown.js"><\/script>$/,
      "<script type=\"text/javascript\" src=\"#{directoryDepth}javascript/ie6dropdown.js\"></script>")

    htmlString.gsub!(/^<link rel="stylesheet" type="text\/css" href=".*style\/ie5.css" media="screen" \/>$/,
      "<link rel=\"stylesheet\" type=\"text/css\" href=\"#{directoryDepth}style\/ie5.css\" media=\"screen\" />")

    htmlString.gsub!(/^<a href=".*impressum.#{MyNavigation.defaultExtension}" class="contact">Impressum<\/a>$/,
      "<a href=\"#{directoryDepth}impressum.#{MyNavigation.defaultExtension}\" class=\"contact\">Impressum</a>")

    htmlString.gsub!(/^<a href=".*sitemap.#{MyNavigation.defaultExtension}" class="sitemap">Sitemap<\/a>$/,
      "<a href=\"#{directoryDepth}sitemap.#{MyNavigation.defaultExtension}\" class=\"sitemap\">Sitemap</a>")

    htmlString.gsub!(/^<form action=".*search.#{MyNavigation.defaultExtension}" method="get">$/,
      "<form action=\"#{directoryDepth}search.#{MyNavigation.defaultExtension}\" method=\"get\">")

    htmlString.gsub!(/^<script type="text\/javascript" src=".*log\/stats.js"><\/script>$/,
      "<script type=\"text/javascript\" src=\"#{directoryDepth}log/stats.js\"></script>")

    htmlString.gsub!(/^<link href=".*favicon.png" type="image\/x-icon" rel="icon" \/>$/,
      "<link href=\"#{directoryDepth}favicon.png\" type=\"image/x-icon\" rel=\"icon\" />")

    htmlString.gsub!(/^ stat\('.*', 'log\/'\);$/,
      " stat('#{file_name}', '#{directoryDepth}log/');")

    return htmlString
  end

end

module TopNavigation

  LINK_SEPARATOR="&raquo;"

  def self.set(htmlString, directoryDepth, file_name)

    # special treatment for imprint
    if MyNavigation.currentElement == nil
      return htmlString
    end

    topNavigation = "<div class=\"topNavigation\">\nDu bist hier:"

    if not (MyNavigation.currentElement.rootElement.name == nil and MyNavigation.currentElement.name == "Start")
      topNavigation <<= "<a href=\"#{directoryDepth}index.#{MyNavigation.defaultExtension}\">Start</a>#{LINK_SEPARATOR}"
    end

    linkArray = []
    parentElement = MyNavigation.currentElement.rootElement

    while parentElement.name != nil
      linkArray << "<a href=\"#{directoryDepth}#{parentElement.parent_directory}#{parentElement.link}\">#{parentElement.name}</a>#{LINK_SEPARATOR}"
      parentElement = parentElement.rootElement
    end

    linkArray.reverse.each { |link|
      topNavigation <<= link
    }

    topNavigation <<= "<span>#{MyNavigation.currentElement.name}"

    # check if file endswith _2.ext :
    if matching = file_name.match(/_([2-9])\.#{MyNavigation.defaultExtension}$/)
      topNavigation <<= " (Teil #{matching.captures[0]})"
    end

    topNavigation <<= "</span>\n</div>"

    htmlString.gsub!(/<div class="topNavigation">.*?<\/div>/m, "#{topNavigation}")

    return htmlString
  end

end


module RelatedLinks

  LINK_SEPARATOR=" |\n"

  def self.getLink(element, directoryDepth)
    linkName = element.name.gsub(/ /, "&nbsp;")
    return "<a href=\"#{directoryDepth}#{element.parent_directory}#{element.link}\">#{linkName}</a>"
  end

  def self.set(htmlString, directoryDepth, file_name)

    relatedLinks = ""

    # if there are no further links, then use links from the same level
    if MyNavigation.currentElement != nil and MyNavigation.currentElement.sub_links == []

      if MyNavigation.currentElement.rootElement.name != nil
        relatedLinks <<= getLink(MyNavigation.currentElement.rootElement, directoryDepth)
      end

      MyNavigation.currentElement.rootElement.sub_links.each { |element|
        if element != MyNavigation.currentElement
          if relatedLinks != ""
            relatedLinks <<= "#{LINK_SEPARATOR}"
          end
          relatedLinks <<= getLink(element, directoryDepth)
        end
      }

    else

      if MyNavigation.currentElement != nil
        MyNavigation.currentElement.sub_links.each { |element|
          if relatedLinks != ""
            relatedLinks <<= "#{LINK_SEPARATOR}"
          end
          relatedLinks <<= getLink(element, directoryDepth)
        }
      end

    end

    keepHTMLReplaceComment = ""

    if matching = htmlString.match(/<div class="relatedLinks">\nweiterführende Links:\n<!-- replace:\n(.*?)-->/m)
      lines = matching.captures[0]
      lineArray = lines.lines.map(&:chomp)
      numberOfReplacements = lineArray.size / 2

      index=0
      numberOfReplacements.times do
        if lineArray[index + 1] == ""
          relatedLinks.sub!(lineArray[index] + "\n", "")
        else
          relatedLinks.sub!(lineArray[index], lineArray[index + 1])
        end
        index = index + 2
      end

      keepHTMLReplaceComment = htmlString.match(/<div class="relatedLinks">\nweiterführende Links:\n(<!-- replace:\n.*?-->\n)/m).captures[0]
    end

    keepHTMLAdditionalComment = ""

    if matching = htmlString.match(/<div class="relatedLinks">\nweiterführende Links:\n.*?\n<!-- additional:(\n.*?)-->/m)
      lines = matching.captures[0]

      keepHTMLAdditionalComment = "&nbsp;|\n" + lines + "<!-- additional:" + lines + "-->"
    end

    htmlString.gsub!(/<div class="relatedLinks">\nweiterführende Links:.*?<\/div>/m,
      "<div class=\"relatedLinks\">\nweiterführende Links:\n#{keepHTMLReplaceComment}#{relatedLinks}#{keepHTMLAdditionalComment}\n</div>")

    return htmlString
  end

end


base_directory=ARGV[0]
file_name=ARGV[1]

savedFilename = "/dev/shm/object_backup.bin"

require 'digest/sha1'

#if not File.exist?(savedFilename)
if true

  navigationRoot = NavigationRoot.new
  navigationRoot.indentation = 2
  MyNavigation.maximum_level = 3

  # navigationRoot.spaces = "  "

  Navigation::loadNavigation(base_directory, "", 1, navigationRoot)

  File.open(savedFilename, 'w') { |f|
    f.write(Marshal.dump(navigationRoot))
  }

else

  sha1sum = Digest::SHA1.file(savedFilename).hexdigest

  navigationRoot = Marshal.load(File.read(savedFilename))
end


Navigation::checkLink(file_name, navigationRoot, 0)


pathname = Pathname.new(file_name)
directoryDepth = 0
while pathname.dirname.to_s != "."
  directoryDepth += 1
  pathname = pathname.dirname
end

directoryLevel = ""

directoryDepth.times do
  directoryLevel <<= "../"
end

navigationRootString = navigationRoot.to_s.gsub(/href="/, "href=\"#{directoryLevel}")

# puts navigationRootString
# exit 0


html_file = File.new(File.join(base_directory, file_name))
htmlString = html_file.read

newNavigationHtmlString = htmlString.sub(/<div id="navigationMenu">.*?<\/div>/m, "<div id=\"navigationMenu\">\n#{navigationRootString}\n </div>")

MyNavigation.maximum_level = 3
MyNavigation.sitemap = true

# add imprint at the bottom
if file_name == "impressum.#{MyNavigation.defaultExtension}"
  navigationRootString = navigationRoot.to_s.
    gsub(/^  <\/ul>$/, "   <li class=\"currentSite\"><a>Impressum</a></li>\n  </ul>").
    gsub(/href="/, "href=\"#{directoryLevel}")
else
  navigationRootString = navigationRoot.to_s.
    gsub(/^  <\/ul>$/, "   <li><a href=\"impressum.#{MyNavigation.defaultExtension}\">Impressum</a></li>\n  </ul>").
    gsub(/href="/, "href=\"#{directoryLevel}")
end


newSitemapHtmlString = newNavigationHtmlString.sub(/<div id="bottomMenuID">.*?<\/div>/m, "<div id=\"bottomMenuID\">\n#{navigationRootString}\n </div>")

correctDirectoriesHtmlString = DirectoryStructure.modifyDirectories(newSitemapHtmlString, directoryLevel, file_name)

topNavigationHtmlString = TopNavigation.set(correctDirectoriesHtmlString, directoryLevel, file_name)

newHtmlString = RelatedLinks.set(topNavigationHtmlString, directoryLevel, file_name)

# write the file finally
puts newHtmlString
